﻿local _, L = ...
local mediaFolder = "Interface\\AddOns\\".._.."\\media\\"
local LMB, RMB = CreateAtlasMarkup("NPE_LeftClick", 16, 16), CreateAtlasMarkup("NPE_RightClick", 16,16)

local LSM = LibStub('LibSharedMedia-3.0')

local max = max
local string_match = string.match
local math_floor = math.floor
local string_len = string.len
local string_gsub = string.gsub
local table_insert = table.insert
local table_sort = table.sort

local CloseLoot = CloseLoot
local CreateFrame = CreateFrame
local CursorOnUpdate = CursorOnUpdate
local CursorUpdate = CursorUpdate
local GetCursorPosition = GetCursorPosition
local GetCVarBool = GetCVarBool
local GetLootSlotInfo = GetLootSlotInfo
local GetLootSlotLink = GetLootSlotLink
local GetLootSlotType = GetLootSlotType
local GetItemCount = GetItemCount
local GetItemInfo = GetItemInfo
local GetNumLootItems = GetNumLootItems
local HandleModifiedItemClick = HandleModifiedItemClick
local IsModifiedClick = IsModifiedClick
local LootSlot = LootSlot
local LootSlotHasItem = LootSlotHasItem
local ResetCursor = ResetCursor
local StaticPopup_Hide = StaticPopup_Hide
local MasterLooterFrame_Show = MasterLooterFrame_Show
local MasterLooterFrame_UpdatePlayers = MasterLooterFrame_UpdatePlayers
local GetContainerNumFreeSlots = GetContainerNumFreeSlots
local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory

local C_Timer_After = C_Timer.After

local ITEM_QUALITY_COLORS = ITEM_QUALITY_COLORS

local info_frame_scale	= 2
local info_frame_space	= 5
local info_frame_width	= 120

lootFrameMixin = {}

local function MultiCheck(check, ...)
	for i = 1, select("#", ...) do
		if check == select(i, ...) then return true end  
	end  
	return false  
end
							
local function CreateMyText(self, font, fontsize, outline, layer, align, justifyH, shadow)
	local text = self:CreateFontString(nil, layer or "OVERLAY", nil, 7)
		text:SetFont(LSM:Fetch(LSM.MediaType.FONT, font) or STANDARD_TEXT_FONT, fontsize or 14, outline or "OUTLINE")
		text:SetPoint(align or "CENTER")
		text:SetJustifyH(justifyH or "CENTER")
		text:SetText(nil)
		text:SetHeight(text:GetStringHeight())
		if shadow then
			text:SetShadowColor(0, 0, 0, .6)
			text:SetShadowOffset(1, -1)
		end	
	return text
end	
	
local function SetMyBorderColor(frame, r, g, b, a)
	frame.top:SetVertexColor(r or 0, g or 0, b or 0, a or 1)
	frame.bottom:SetVertexColor(r or 0, g or 0, b or 0, a or 1)
	frame.left:SetVertexColor(r or 0, g or 0, b or 0, a or 1)
	frame.right:SetVertexColor(r or 0, g or 0, b or 0, a or 1)
end

local function CreateMyBackdrop(parent, pixelSize, r_br, g_br, b_br, a_br, bgTexture, r_bg, g_bg, b_bg, a_bg, shadow_offset, r_s, g_s, b_s, a_s)
	local frame = CreateFrame("Frame", nil, parent)
	frame:SetFrameLevel(parent:GetFrameLevel() - 1)
	frame:ClearAllPoints()
	frame:SetAllPoints(parent)
			
	frame.top = frame:CreateTexture(nil, "OVERLAY")
	frame.top:ClearAllPoints()
	PixelUtil.SetPoint(frame.top, "BOTTOMLEFT", frame, "TOPLEFT", 0, 0)
	PixelUtil.SetPoint(frame.top, "BOTTOMRIGHT", frame, "TOPRIGHT", 0, 0)
	PixelUtil.SetHeight(frame.top, pixelSize, 1)
	frame.top:SetTexture("Interface\\Buttons\\WHITE8x8")
	frame.top:Show()
	
	frame.bottom = frame:CreateTexture(nil, "OVERLAY")
	frame.bottom:ClearAllPoints()
	PixelUtil.SetPoint(frame.bottom, "TOPLEFT", frame, "BOTTOMLEFT", 0, 0)
	PixelUtil.SetPoint(frame.bottom, "TOPRIGHT", frame, "BOTTOMRIGHT", 0, 0)
	PixelUtil.SetHeight(frame.bottom, pixelSize, 1)
	frame.bottom:SetTexture("Interface\\Buttons\\WHITE8x8")
	frame.bottom:Show()
	
	frame.left = frame:CreateTexture(nil, "OVERLAY")
	frame.left:ClearAllPoints()
	PixelUtil.SetPoint(frame.left, "TOPRIGHT", frame, "TOPLEFT", 0, pixelSize)
	PixelUtil.SetPoint(frame.left, "BOTTOMRIGHT", frame, "BOTTOMLEFT", 0, -pixelSize)
	PixelUtil.SetWidth(frame.left, pixelSize, 1)
	frame.left:SetTexture("Interface\\Buttons\\WHITE8x8")
	frame.left:Show()
	
	frame.right = frame:CreateTexture(nil, "OVERLAY")
	frame.right:ClearAllPoints()
	PixelUtil.SetPoint(frame.right, "TOPLEFT", frame, "TOPRIGHT", 0, pixelSize)
	PixelUtil.SetPoint(frame.right, "BOTTOMLEFT", frame, "BOTTOMRIGHT", 0, -pixelSize)
	PixelUtil.SetWidth(frame.right, pixelSize, 1)
	frame.right:SetTexture("Interface\\Buttons\\WHITE8x8")
	frame.right:Show()
	
	SetMyBorderColor(frame, r_br, g_br, b_br, a_br)
	if bgTexture then
		frame.bg = frame:CreateTexture(nil, "BACKGROUND")
		frame.bg:ClearAllPoints()
		frame.bg:SetAllPoints(frame)
		frame.bg:SetTexture(bgTexture)
		frame.bg:SetVertexColor(r_bg / 3 or 0, g_bg / 3 or 0, b_bg / 3 or 0, a_bg or 0)
	end
		
	if shadow_offset then
		frame.shadow = CreateFrame("Frame", nil, frame, "BackdropTemplate")
		frame.shadow:SetFrameLevel(parent:GetFrameLevel() - 1)
		frame.shadow:ClearAllPoints()
		frame.shadow:SetPoint("TOPLEFT", frame, "TOPLEFT", -shadow_offset, shadow_offset)
		frame.shadow:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", shadow_offset, -shadow_offset)
		frame.shadow:SetBackdrop({bgFile = nil, edgeFile = mediaFolder.."glowTex", edgeSize = shadow_offset, tile = false, tileSize = 0, insets = 1})
		frame.shadow:SetBackdropBorderColor(r_s, g_s, b_s, a_s)
		frame.shadow:Hide()
	end
	
	return frame
end

local function SetFrameScale(frame, width, height, from, to)
	if (frame.currentScale ~= to) then
		frame:SetSize(width * from, height * from)
		if (frame.scale:IsPlaying()) then
			frame.scale:Stop()
		end
		frame.scale.width:SetChange(width * to)
		frame.scale.height:SetChange(height * to)
		frame.scale:Play()
		frame.currentScale = to
	end
end

local function ResetInfoFrameFunction(framePool, frame)
	if frame.AnimOut:IsPlaying() then
		frame.AnimOut:Stop()
	end
	if frame.AnimIn:IsPlaying() then
		frame.AnimIn:Stop()
	end
	if frame.AnimDown:IsPlaying() then
		frame.AnimDown:Stop()
	end
	frame:SetAlpha(0)
	frame:Hide()
	frame:SetParent(aLootInfoFrame)
	frame:ClearAllPoints()
	frame:SetPoint("TOP", aLootInfoFrame, "TOP", 0, 0)
	--frame.Text:ClearAllPoints()
	--frame.Text:SetPoint("RIGHT", frame, "RIGHT", -13, 0)
	frame.AnimIn.Alpha:SetDuration(L.config.info_frame_speed)
	frame.AnimDown.Translation:SetDuration(L.config.info_frame_speed)
	frame.createdAt = nil
	frame.expired = false
end

function lootFrameMixin:AddFrameToPool(icon, text, fast)
	local lastFrame = aLootInfoFrame.infoFrameActiveFrames.last
	if lastFrame ~= nil and lastFrame.AnimIn:IsPlaying() then
		C_Timer_After(lastFrame.AnimIn:GetDuration(), function()
			if lastFrame ~= nil then
				lootFrameMixin:AddFrameToPool(icon, text, true)
			end
		end)
		return
	end
		
	aLootInfoFrame:Show()
	aLootInfoFrame:EnableMouse(false)
	
	-- hide moveable frame ak je otvoreny
	aLootInfoFrame.DragBar:EnableMouse(false)
	if L.config.unlock_infoFrame then
		aLootInfoFrame.Text:SetText(nil)
		aLootInfoFrame:SetBackdropColor(0, 0, 0, 0)
		aLootInfoFrame:SetBackdropBorderColor(0, 0, 0, 0)
		L.config.unlock_infoFrame = false
		lootFrameMixin:UpdateNotifyChange("aLootInfoFrame", false)
	end
	
	aLootInfoFrame:SetHeight(L.config.info_frame_height + 10)
	
	if aLootInfoFrame:GetScript("OnUpdate") == nil then
		aLootInfoFrame:SetScript("OnUpdate", function(self, elapsed)
			if self.infoFrameActiveFrames:GetNumActive() > 0 then
				self:Show()
				
				local sortedFrames = {}
				for frame in self.infoFrameActiveFrames:EnumerateActive() do
					table_insert(sortedFrames, frame)
				end
				table_sort(sortedFrames, function(a, b) return a.createdAt < b.createdAt end)
				
				for _, frame in ipairs(sortedFrames) do
					if GetTime() >= (frame.createdAt + L.config.info_frame_duration) and not (frame.AnimOut:IsPlaying() or frame.expired) then
						frame.AnimOut:Play()
						frame.AnimOut:SetScript("OnFinished", function(self)
							self:GetParent().expired = true
						end)
					end
					
					if self.infoFrameActiveFrames:GetNumActive() > L.config.info_frame_maxcount or frame.expired then
						frame.Text:SetText(nil)
						frame.Text:Hide()
						frame:Hide()
						self.infoFrameActiveFrames:Release(frame)
					end
				end
			else
				aLootInfoFrame.infoFrameActiveFrames:ReleaseAll()
				aLootInfoFrame.infoFrameActiveFrames.last = nil
				if not L.config.unlock_infoFrame then
					aLootInfoFrame:Hide()
				end
				aLootInfoFrame:SetScript("OnUpdate", nil)
			end
		end)
	end
	
	local newFrame = aLootInfoFrame.infoFrameActiveFrames:Acquire()
	newFrame:SetPoint("BOTTOMLEFT", aLootInfoFrame, "BOTTOMLEFT", 0, 0)
	newFrame:EnableMouse(false)
	newFrame.createdAt = GetTime()
	
	newFrame.IconFrame:SetSize(L.config.info_frame_height + 20, L.config.info_frame_height + 20)	
	newFrame.IconFrame.Icon:SetSize(L.config.info_frame_height, L.config.info_frame_height)
	newFrame.IconFrame.Icon:SetTexture(icon)
		
	--[[newFrame.Text:SetFont(LSM:Fetch(LSM.MediaType.FONT, L.config.info_frame_font) or STANDARD_TEXT_FONT, L.config.info_frame_fontsize or 14, L.config.info_frame_font_flag or "OUTLINE")
	newFrame.Text:SetPoint("RIGHT", newFrame, "RIGHT", -13, 0)
	newFrame.Text:SetText(text)
	newFrame.Text:Show()
	]]--
	
	newFrame.Text = CreateMyText(newFrame, L.config.info_frame_font, L.config.info_frame_fontsize, L.config.info_frame_font_flag, "OVERLAY", "CENTER", "CENTER", true)
	newFrame.Text:SetPoint("RIGHT", newFrame, "RIGHT", -13, 0)
	newFrame.Text:SetText(text)
		
	newFrame.currentScale = info_frame_scale
	newFrame.scale = CreateAnimationGroup(newFrame)
	newFrame.scale.width = newFrame.scale:CreateAnimation("Width")
	newFrame.scale.width:SetDuration(L.config.info_frame_speed)
	newFrame.scale.height = newFrame.scale:CreateAnimation("Height")
	newFrame.scale.height:SetDuration(L.config.info_frame_speed)
			
	-- AnimeIn
	if fast then
		newFrame.AnimIn.Alpha:SetDuration(L.config.info_frame_speed / 2)
		newFrame.scale.width:SetDuration(L.config.info_frame_speed/2)
		newFrame.scale.height:SetDuration(L.config.info_frame_speed/2)
	end
		
	SetFrameScale(newFrame, newFrame.Text:GetStringWidth() + L.config.info_frame_height + 40, L.config.info_frame_height + 10, info_frame_scale, 1)
	
	--newFrame.AnimIn.Alpha:SetFromAlpha(0)
	newFrame.AnimIn.Alpha:SetToAlpha(L.config.info_frame_alpha)
	newFrame.AnimIn:Play()
	
	newFrame.AnimIn:SetScript("OnFinished", function(self)
		self:GetParent():SetAlpha(L.config.info_frame_alpha)
	end)
	
	if newFrame.AnimIn:IsPlaying() then
		newFrame.AnimIn:GetParent():Show()
	end

	if L.config.info_frame_flash then
		--newFrame.IconFrame.glow:SetVertexColor(1, 1, 1, 1)
		newFrame.IconFrame.glow:Show()
		newFrame.IconFrame.glow.animIn:Play()
	end
	--newFrame.glow:Show()
	--newFrame.glow.animIn:Play()
	
	newFrame.AnimOut.Alpha:SetFromAlpha(L.config.info_frame_alpha)
	--newFrame.AnimOut.Alpha:SetToAlpha(0)	
	
	-- Anime move
	if lastFrame ~= nil and lastFrame ~= newFrame and not lastFrame.AnimOut:IsPlaying() then
		if fast then
			lastFrame.AnimDown.Translation:SetDuration(L.config.info_frame_speed / 2)
		end
		lastFrame.AnimDown.Translation:SetOffset(0, - L.config.info_frame_height - (info_frame_space + 2))
		lastFrame.AnimDown:Play()
		lastFrame.AnimDown:SetScript("OnFinished", function(self)
			self:GetParent():ClearAllPoints()
			self:GetParent():SetParent(newFrame)
			self:GetParent():SetPoint("BOTTOMLEFT", newFrame, "BOTTOMLEFT", 0, - L.config.info_frame_height - (info_frame_space + 2))
			self:GetParent():SetAlpha(1)
		end)
	end

	aLootInfoFrame.infoFrameActiveFrames.last = newFrame
end

function lootFrameMixin:ShowFlashFrame(icon, lootName)
	if aLootFlashFrame.Anime:IsPlaying() then
		aLootFlashFrame.Anime:Stop()
	end
		
	aLootFlashFrame.DragBar:EnableMouse(false)
	aLootFlashFrame.Text:SetFont(LSM:Fetch(LSM.MediaType.FONT, L.config.flash_frame_font) or STANDARD_TEXT_FONT, L.config.flash_frame_fontsize or 14, L.config.flash_frame_font_flag or "OUTLINE")
	aLootFlashFrame.Text:SetText(lootName)
	aLootFlashFrame:SetSize(aLootFlashFrame.Text:GetStringWidth() + 90, L.config.flash_frame_height)
	aLootFlashFrameIconFrame.Icon:SetTexture(icon or 939375)
	aLootFlashFrame:Show()
	aLootFlashFrame.Anime.AlphaIn:SetToAlpha(L.config.flash_frame_alpha)
	aLootFlashFrame.Anime:Play()
	
	if L.config.flash_frame_flash then
		aLootFlashFrame.shine:Show()
		aLootFlashFrame.shine.animIn:Play()
		aLootFlashFrame.glow:SetVertexColor(.5, .5, .5, 1)
		aLootFlashFrame.glow:Show()
		aLootFlashFrame.glow.animIn:Play()
	end
end

local function OnEnter(self)
	if L.config.toolTip then
		local slotIndex = self:GetID()
		local slotType = GetLootSlotType(slotIndex) -- LOOT_SLOT_NONE, LOOT_SLOT_ITEM, LOOT_SLOT_MONEY, LOOT_SLOT_CURRENCY 
		
		if ( slotType == LOOT_SLOT_CURRENCY ) then
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 10, -10)
			GameTooltip:SetLootCurrency(slotIndex)
			CursorUpdate(self)
		else
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 10, -10)
			GameTooltip:SetLootItem(slotIndex)
			CursorUpdate(self)
		end
	end
	
	if L.config.name_show and L.config.name_show_onlyMO then
		self.nameTexture:Show()
		self.name:Show()
	end
	if L.config.mouseover_scale > 1 then
		SetFrameScale(self, L.config.iconSize, L.config.iconSize, 1, L.config.mouseover_scale)
	end
	if L.config.mouseover_color then
		self.border.shadow:SetBackdropBorderColor(unpack(L.config.mouseover_color_value))
	end
end

local function OnLeave(self)
	if L.config.toolTip then
		GameTooltip:Hide()
	end
	
	if L.config.name_show_onlyMO then
		self.nameTexture:Hide()
		self.name:Hide()
	end
		
	if L.config.mouseover_scale > 1 then
		SetFrameScale(self, L.config.iconSize, L.config.iconSize, L.config.mouseover_scale, 1)
	end
	
	if self.lootQuality > 1 then
		self.border.shadow:SetBackdropBorderColor(self.color.r, self.color.g, self.color.b)
	else
		self.border.shadow:SetBackdropBorderColor(0, 0, 0)
	end
	
	ResetCursor()
end

local function OnClick(self, button)
	if button == "RightButton" then
		ToggleAllBags()
	else
		if self.lootName == "Test" then
			lootFrameMixin:UpdateNotifyChange("openLoot", true)
		elseif IsModifiedClick() then
			HandleModifiedItemClick(GetLootSlotLink(self:GetID()))
		else
			StaticPopup_Hide('CONFIRM_LOOT_DISTRIBUTION')
			LootSlot(self:GetID())
		end
	end
end

local function OnShow(self)
	if GameTooltip:IsOwned(self) then
		GameTooltip:SetOwner(self,"ANCHOR_BOTTOMRIGHT", 10, -10)
		GameTooltip:SetLootItem(self:GetID())
		CursorOnUpdate(self)
	end
end

local function OnLootClosed()
	if aLootFrame:IsShown() then
		StaticPopup_Hide('LOOT_BIND')
		aLootFrame:Hide()
		
		-- pre istotu Hide na vsetky sloty
		for _, frame in pairs(aLootFrame.slots) do
			frame:Hide()
		end
	end		
	
	if L.config.infoFrame and L.config.info_frame_showBagSpace and aLootFrame.lootReadyIsOpened then
		local totalFree = CalculateTotalNumberOfFreeBagSlots()
		aLootFrame.lootReadyIsOpened = false
				
		if totalFree == 0 then
			lootFrameMixin:AddFrameToPool(133670, "|cffff0000"..ERR_INV_FULL.."|r")
		elseif totalFree == 1 then
			lootFrameMixin:AddFrameToPool(133670, "|cffff0000Last free slot|r")
		elseif totalFree < L.config.info_frame_numBagSpace then
			lootFrameMixin:AddFrameToPool(133670, "|cffff0000"..totalFree.." free slots in bag|r")
		end
	end

--[[
	if not aLootFrame.isFromItem then
		CloseAllBags()
	else
		lootFrameMixin:ShowFlashFrame(nil, "isFromItem")
	end
]]--
end

local function OnLootReady(autoloot)
	if not aLootFrame.lootReadyIsOpened then
		local numLootItems = GetNumLootItems()
		aLootFrame.lootReadyIsOpened = true
				
		if numLootItems > 0 then
			for i = numLootItems, 1, -1 do
				LootSlot(i)
			end

			if L.config.infoFrame and  L.config.info_frame_speedLoot_text then
				lootFrameMixin:AddFrameToPool(133784, "|cff00bbffLooted|r "..numLootItems.." |cff00bbffitems|r")
			end
		end
	end
end

-- uprav zarovnanie pri zmene slot itemov
local function anchorSlots()
	local shownSlots, firstSlot, lastSlot = 0
	
	for i = 1, #aLootFrame.slots do
		aLootFrame.slots[i]:ClearAllPoints()
		if aLootFrame.slots[i]:IsShown() then
			shownSlots = shownSlots + 1
			if not firstSlot then
				firstSlot = i
			end
			if shownSlots > 1 then
				if L.config.lootFrame_align == "LEFT" then
					aLootFrame.slots[i]:SetPoint("LEFT", lastSlot, "RIGHT", L.config.iconSpace, 0)
				elseif L.config.lootFrame_align == "RIGHT" then
					aLootFrame.slots[i]:SetPoint("RIGHT", lastSlot, "LEFT", - L.config.iconSpace, 0)
				elseif L.config.lootFrame_align == "TOP" then
					aLootFrame.slots[i]:SetPoint("TOP", lastSlot, "BOTTOM", 0, - L.config.iconSpace)
				elseif L.config.lootFrame_align == "BOTTOM" then
					aLootFrame.slots[i]:SetPoint("BOTTOM", lastSlot, "TOP", 0, L.config.iconSpace)
				else
					aLootFrame.slots[i]:SetPoint("LEFT", lastSlot, "RIGHT", L.config.iconSpace, 0)
				end
			end
			lastSlot = aLootFrame.slots[i]
		end
	end
	
	if shownSlots > 0 then
		if C_CVar.GetCVarBool("lootUnderMouse") or L.config.lootFrame_align ~= "CENTER" then
			aLootFrame.slots[firstSlot]:SetPoint(L.config.lootFrame_align, aLootFrame, L.config.lootFrame_align, 0, 0)
		else
			aLootFrame.slots[firstSlot]:SetPoint("CENTER", aLootFrame, "CENTER", -(((L.config.iconSize + L.config.iconSpace) * (shownSlots - 1)) / 2), 0)
		end
	end
end

local function createSlot(id)
	local frame = CreateFrame("Button", 'aLootFrameSlot'..id, aLootFrame, "aLootFrameSlot")
		frame:SetFrameLevel(aLootFrame:GetFrameLevel() + 1)
		frame:SetPoint("CENTER", 0, 0)
		frame:SetSize(L.config.iconSize, L.config.iconSize)
		frame:SetID(id)
	
		frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")

		frame:SetScript("OnEnter", OnEnter)
		frame:SetScript("OnLeave", OnLeave)
		frame:SetScript("OnClick", OnClick)
		frame:SetScript("OnShow", OnShow)
	
		frame.border = CreateMyBackdrop(frame, 1, 0, 0, 0, 1,"Interface\\Buttons\\WHITE8x8", 0, 0, 0, .7, 5, 0, 0, 0, 0.6)
		frame.border.shadow:Show()
				
		--frame.currentScale = 1
		frame.scale = CreateAnimationGroup(frame)
		frame.scale.width = frame.scale:CreateAnimation("Width")
		frame.scale.width:SetDuration(.3)
		frame.scale.height = frame.scale:CreateAnimation("Height")
		frame.scale.height:SetDuration(.3)
	
	local iconFrame = CreateFrame("Frame", nil, frame)
		--iconFrame:SetSize(L.config.iconSize, L.config.iconSize)
		iconFrame:SetAllPoints()
		frame.iconFrame = iconFrame
		
		frame.nameTexture = frame.iconFrame:CreateTexture(nil, "OVERLAY", nil, 6)
		frame.nameTexture:SetTexture(LSM:Fetch(LSM.MediaType.STATUSBAR, L.config.name_texture))
		frame.nameTexture:Hide()
		
		frame.name = CreateMyText(frame.iconFrame, L.config.name_font, L.config.name_fontsize, L.config.name_font_flag, "OVERLAY", "CENTER", "CENTER", false)
		frame.name:SetText(nil)
		frame.name:Hide()
		
		frame.icon = iconFrame:CreateTexture(nil, "OVERLAY")
		frame.icon:SetAlpha(.8)
		frame.icon:SetTexCoord(.07, .93, .07, .93)
		frame.icon:SetAllPoints()
	
		frame.count = CreateMyText(iconFrame, L.config.name_font, 16, "OUTLINE", "OVERLAY", "CENTER", "RIGHT", false)
		frame.count:SetPoint("BOTTOMRIGHT", iconFrame, -1, 2)
		frame.count:SetText(nil)
		
		frame.questTexture = iconFrame:CreateTexture(nil, 'OVERLAY')
		frame.questTexture:SetPoint("TOPRIGHT", iconFrame, "TOPRIGHT", 4, 0)
		frame.questTexture:SetSize(16, 24)
		frame.questTexture:SetAtlas("questlegendary")
		frame.questTexture:Hide()
	
	aLootFrame.slots[id] = frame
	return frame
end

local function CheckFreeSpace(itemLink, lootQuantity)
	local itemFamily = GetItemFamily(itemLink)
	local itemCount = GetItemCount(itemLink)
	
	for bagID = BACKPACK_CONTAINER, NUM_BAG_SLOTS do
		local numberOfFreeSlots, bagType = GetContainerNumFreeSlots(bagID)
		if (not bagType or bagType == 0) or (itemFamily and bit.band(itemFamily, bagType) > 0) then
			if numberOfFreeSlots > 0 then
				return true, numberOfFreeSlots, itemCount
			end
		end
	end
	
	if itemCount > 0 then
		local itemStackCount = select(8, GetItemInfo(itemLink))
		if itemStackCount > 1 then
			local remainingSpace = (itemStackCount - itemCount) % itemStackCount
			if remainingSpace >= lootQuantity then
				return true, numberOfFreeSlots, itemCount
			end
		end
	end

	return false, 0, 0
end

local function OpenSlot(slotIndex, test)
	if LootSlotHasItem(slotIndex) or test then
		--[[
		itemName, slot.itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemID or "itemString" or "itemName" or "slot.itemLink")
		slot.lootName, lootIcon, slot.lootQuantity, slot.lootQuality = CurrencyContainerUtil.GetCurrencyContainerInfo(slot.currencyID, slot.lootQuantity, slot.lootName, slot.lootIcon, slot.lootQuality)
		slot.lootQuality = 0 Poor, 1 Common, 2 Uncommon, 3 Rare, 4 Epic, 5 Legendary, 6 Artifact
		slotType 0: LOOT_SLOT_NONE, 1: LOOT_SLOT_ITEM, 2 LOOT_SLOT_MONEY, 3 LOOT_SLOT_CURRENCY
		]]--
		local isFreeSpaceInBags, numberOfFreeSlots, itemCount
		local slot = aLootFrame.slots[slotIndex] or createSlot(slotIndex)
		
		if test then
			slot.lootIcon, slot.lootName, slot.lootQuantity, slot.currencyID, slot.lootQuality, slot.locked, slot.isQuestItem, slot.questID, slot.isActive = 4287471, "Test", 3, nil, 2, false, false, nil, nil
			slot.slotType = 1
			slot.itemLink = select(2, GetItemInfo(slot.lootName))
			aLootFrame.numLootItems = 1
		else
			slot.lootIcon, slot.lootName, slot.lootQuantity, slot.currencyID, slot.lootQuality, slot.locked, slot.isQuestItem, slot.questID, slot.isActive = GetLootSlotInfo(slotIndex)
			slot.slotType = GetLootSlotType(slotIndex)	
			slot.itemLink = GetLootSlotLink(slotIndex)
		end
				
		slot.currentScale = 1
		slot.questTexture:Hide()
		
		if slot.locked then
			slot.color = { r = 1, g = 0, b = 0, hex = "|cFFFF0000" }
		elseif (slot.questID and not slot.isActive) or (slot.questID or slot.isQuestItem) then
			slot.color = { r = 1, g = 1, b = 0, hex = "|cFFFFFF00" }
			slot.questTexture:Show()
		elseif (slot.questID or slot.isQuestItem) then
			slot.color = { r = 1, g = 1, b = 0, hex = "|cFFFFFF00" }
			slot.questTexture:Show()
		elseif slot.lootQuality > 1 then
			slot.color = ITEM_QUALITY_COLORS[slot.lootQuality]
		else
			slot.color = { r = 0, g = 0, b = 0, hex = ITEM_QUALITY_COLORS[slot.lootQuality].hex }
		end
		
		-- goldy, questItems a currency sa nelootuje do baglu preto automaticky je miesto
		if test then
			isFreeSpaceInBags = true
		elseif slot.slotType == 2 or slot.isQuestItem then
			isFreeSpaceInBags = true
		elseif slot.slotType == 3 then
			local info = C_CurrencyInfo.GetCurrencyInfo(slot.currencyID)
			itemCount = info.quantity
			isFreeSpaceInBags = true
		else
			isFreeSpaceInBags, numberOfFreeSlots, itemCount = CheckFreeSpace(slot.itemLink, slot.lootQuantity)
		end
		
		if isFreeSpaceInBags and not test and
			((C_CVar.GetCVarBool("autoLootDefault") and not IsModifiedClick("AUTOLOOTTOGGLE")) 
			or (L.config.autoLoot["gold"] and slot.slotType == 2) 
			or (L.config.autoLoot["currency"] and slot.slotType == 3)
			or (L.config.autoLoot["quest"] and (slot.isQuestItem or slot.questID))
			or (L.config.autoLoot["poor"] and slot.lootQuality == 0)
			or (L.config.autoLoot["common"] and slot.lootQuality == 1)
			or (L.config.autoLoot["uncommon"] and slot.lootQuality == 2)
			or (L.config.autoLoot["rare"] and slot.lootQuality == 3)
			or (L.config.autoLoot["epic"] and slot.lootQuality == 4)
			or (L.config.autoLoot["legendary"] and slot.lootQuality == 5)) then
			
			LootSlot(slotIndex)
						
			if slot.slotType == 2 then
				slot.lootName = string_gsub(slot.lootName, "\n", " ")
			end
			
			if L.config.flashFrame 
				and ((L.config.flash["user"] and L.ItemsTable[slot.lootName] and L.ItemsTable[slot.lootName].enable)
					or (L.config.flash["quest"] and (slot.isQuestItem or slot.questID))
					or (L.config.flash["legendary"] and slot.lootQuality == 5)) then
				lootFrameMixin:ShowFlashFrame(slot.lootIcon, slot.color.hex .. slot.lootName.."|r")
				PlaySoundFile(LSM:Fetch(LSM.MediaType.SOUND, L.config.flash_sound), "Master")
			end
			
			if L.config.infoFrameBlizzard then
				if slot.slotType == 2 then
					-- retardovany string
					--MoneyWonAlertSystem:AddAlert(slot.lootName)
				elseif slot.slotType == 3 then
					--slot.itemLink, originalQuantity, rollType, roll, specID, isCurrency, showFactionBG, lootSource, lessAwesome, isUpgraded, isCorrupted, wonRoll, showRatedBG, isSecondaryResult
					LootAlertSystem:AddAlert(slot.itemLink, slot.lootQuantity, nil, nil, nil, true);
				else
					LootAlertSystem:AddAlert(slot.itemLink, slot.lootQuantity, nil, nil, nil, false);
				end
			end
				
			
			if L.config.infoFrame then
				local text = ''
				if L.config.info_frame_showLootQuantity and slot.lootQuantity and slot.lootQuantity > 1 then
					text = text .. "|cffFFFFFF x"..slot.lootQuantity.."|r"
				end
			
				if  L.config.info_frame_showItemCount and itemCount and itemCount > 1 then
					text = text .. " |cff808080("..itemCount..")|r"
				end
				
				lootFrameMixin:AddFrameToPool(slot.lootIcon, slot.color.hex .. slot.lootName  .."|r"..text)
			end
					
			return
		elseif (L.config.flashFrame and not isFreeSpaceInBags) then
			if not aLootFlashFrame.Anime:IsPlaying() then
				lootFrameMixin:ShowFlashFrame(nil, "|cffff0000" .. ERR_INV_FULL  .."|r")
				PlaySoundFile(LSM:Fetch(LSM.MediaType.SOUND, L.config.flash_error_sound), "Master")
			end
		end
		
		-- nebolo nic lootnute zobraz loot
		if slot.lootIcon then
			aLootFrame:Show()
			aLootFrame.DragBar:EnableMouse(false)
			
			if slot.FadeIn:IsPlaying() then
				slot.FadeIn:Stop()
			end
								
			-- create name			
			if L.config.name_short and (string_len(slot.lootName) > L.config.name_short_lenght) then
				if L.config.name_format == 0 then
					slot.lootName =  string_gsub(slot.lootName, "%s?(.[\128-\191]*)%S+%s", "%1. ")
				elseif L.config.name_format == 1 then
					slot.lootName = string_gsub(slot.lootName, "%s?(.[\128-\191]*)%S+%s", "%1.")
				elseif L.config.name_format == 2 then
					slot.lootName =  string_gsub(slot.lootName, "%s(%a)%a*", " %1.")
				elseif L.config.name_format == 3 then
					slot.lootName =  string_gsub(slot.lootName, "%s(%a)%a*", "%1.")
				elseif L.config.name_format == 4 then
					for n in string_gmatch(slot.lootName, '%S+') do
						slot.lootName = n
						break
					end
				elseif L.config.name_format == 5 then
					for n in string_gmatch(slot.lootName, '%S+') do
						slot.lootName = n
					end
				end
			end
			
			
			if L.config.name_color then
				slot.name:SetText(slot.lootName)
				slot.name:SetTextColor(unpack(L.config.name_font_color))
			else
				--slot.name:SetTextColor(slot.color.r, slot.color.g, slot.color.b)
				slot.name:SetText(slot.color.hex .. slot.lootName .."|r")
			end
			
			slot.nameTexture:ClearAllPoints()
			slot.nameTexture:SetPoint(L.config.name_font_point, slot.iconFrame, L.config.name_font_relativePoint, L.config.name_font_posun_X, L.config.name_font_posun_Y)
			slot.nameTexture:SetSize(slot.name:GetStringWidth() + 10, L.config.name_fontsize + 4)
			slot.name:SetAllPoints(slot.nameTexture)
						
			if L.config.name_texture_color then
				slot.nameTexture:SetVertexColor(unpack(L.config.name_texture_color_value))		
			else
				slot.nameTexture:SetVertexColor(slot.color.r, slot.color.g, slot.color.b, .3)
			end
						
			if L.config.name_show and not L.config.name_show_onlyMO then
				slot.nameTexture:Show()
				slot.name:Show()
			else
				slot.nameTexture:Hide()
				slot.name:Hide()
			end	
					
			if slot.lootQuantity and (slot.lootQuantity > 1) then
				slot.count:SetText(slot.lootQuantity)
				slot.count:Show()
			elseif slot.slotType == 2 then
				local coins = string_match(slot.lootName, "(%d+)")
				coins = tonumber(coins)
				if coins and (coins > 0) then
					slot.count:SetText(coins)
					slot.count:Show()
				end
			else
				slot.count:Hide()
			end
							
			slot.icon:SetTexture(slot.lootIcon)
			slot.border.shadow:SetBackdropBorderColor(slot.color.r, slot.color.g, slot.color.b)
			slot:Enable()	
		else
			slot.icon:SetTexture("Interface\\Icons\\INV_Misc_Herb_AncientLichen")
			slot.count:Hide()
			slot.border.shadow:Hide()
			slot:Disable()	
		end
		
		slot:Show()
		slot.border:Show()
		slot.FadeIn:Play()
		
	elseif aLootFrame.slots[slotIndex] then
		aLootFrame.slots[slotIndex]:Hide()
	end
end

local function OnLootOpened(...)
	local autoLoot, isFromItem = ...
				
--	aLootFrame:Show()
	if ( isFromItem ) then
		aLootFrame.isFromItem = isFromItem
		PlaySound(SOUNDKIT.UI_CONTAINER_ITEM_OPEN)
	end
	
	-- ak je zapnuty loot na mieste kurzora
	if C_CVar.GetCVarBool("lootUnderMouse") then
		local x, y = GetCursorPosition()
		x = x / aLootFrame:GetEffectiveScale()
		y = y / aLootFrame:GetEffectiveScale()
		aLootFrame:ClearAllPoints()
		aLootFrame:SetPoint("TOPLEFT", nil, "BOTTOMLEFT", x - (L.config.iconSize / 2), y + (L.config.iconSize / 2))
		aLootFrame:GetCenter()
		aLootFrame:Raise()	
	end
	
	aLootFrame.lootReadyIsOpened = true
	aLootFrame.numLootItems = GetNumLootItems()
	
	if aLootFrame.numLootItems > 0 then
		for i = 1, aLootFrame.numLootItems do
			OpenSlot(i)
			--SetFrameScale(aLootFrame.slots[i], L.config.iconSize, L.config.iconSize, 1.4, 1)
		end
	end
	
	anchorSlots()
end

local function OnLootSlotCleared(...)
	local slot = ...
	
	if aLootFrame:IsShown() and aLootFrame.slots[slot] and aLootFrame.slots[slot]:IsShown() then
		aLootFrame.slots[slot]:Hide()
		anchorSlots()
	end
end

function OnShowLootFrameBase(self)
	local numLootItems = GetNumLootItems()
	if( numLootItems == 0 ) then
		PlaySound(SOUNDKIT.LOOT_WINDOW_OPEN_EMPTY)
	elseif( IsFishingLoot() ) then
		PlaySound(SOUNDKIT.FISHING_REEL_IN)
	else
		PlaySound(SOUNDKIT.SOULBINDS_OPEN_UI)
	end
end


function OnHideLootFrameBase(self)
	CloseLoot()
	--StaticPopup_Hide('CONFIRM_LOOT_DISTRIBUTION')
	--MasterLooterFrame:Hide()	
end

local function OnAddonLoaded(self, ...)
	local name = ...
	if name == _ then
		if not _G["aLootCFG"] then _G["aLootCFG"] = L.defaults end
		L.config = _G["aLootCFG"]

		for key, value in pairs(L.defaults) do
			if L.config[key] == nil then
				L.config[key] = value
			end
		end
		
		L.ItemsTable = ItemsTable or {}
		ItemsTable = L.ItemsTable
		
		LibStub('AceConfigRegistry-3.0'):RegisterOptionsTable(_, L.options)
		L.basic_page = LibStub('AceConfigDialog-3.0'):AddToBlizOptions(_, "aLoot", nil, "basic_page")
		
		-- lootFrame				
		L.config.unlock_lootFrame = false
		aLootFrame.DragBar:EnableMouse(false)
		aLootFrame.Text:SetText(nil)
		aLootFrame:SetBackdropColor(0, 0, 0, 0)
		aLootFrame:SetBackdropBorderColor(0, 0, 0, 0)
		
		aLootFrame.slots = {}
		aLootFrame:SetScript("OnShow", OnShowLootFrameBase)
		aLootFrame:SetScript("OnHide", OnHideLootFrameBase)
		
		_G.LootFrame:UnregisterAllEvents()
		_G.AlertFrame:UnregisterEvent("SHOW_LOOT_TOAST")
		
		table_insert(_G.UISpecialFrames, "aLootFrame")
		hooksecurefunc(_G.MasterLooterFrame, 'Hide', _G.MasterLooterFrame.ClearAllPoints)
		
		-- infoFrame
		L.config.unlock_infoFrame = false
		aLootInfoFrame.Text:SetText(nil)
		aLootInfoFrame:SetBackdropColor(0, 0, 0, 0)
		aLootInfoFrame:SetBackdropBorderColor(0, 0, 0, 0)
				
		aLootInfoFrame.infoFrameActiveFrames = CreateFramePool("Button", infoFrameBase, "aLootInfoFramePool", ResetInfoFrameFunction)
		aLootInfoFrame.infoFrameActiveFrames.last = nil
					
		-- flashFrame
		L.config.unlock_flashFrame = false
		aLootFlashFrame.DragBar:EnableMouse(false)		
		
		-- minimap icon
		L.miniMapIconSave = miniMapIconSave or {}
		miniMapIconSave = L.miniMapIconSave
		
		local LDB = LibStub("LibDataBroker-1.1"):NewDataObject( _ , {
			type = "data source",
			text = "aLoot Addon",
			icon = 4359252,
			OnClick = function(self, button)
				if _G.InterfaceOptionsFrame:IsShown() then
					_G.InterfaceOptionsFrame:Hide()
				else
					InterfaceOptionsFrame_OpenToCategory("aLoot")
					InterfaceOptionsFrame_OpenToCategory("aLoot")
				end
			end,
			OnTooltipShow = function(tooltip)
				if not tooltip or not tooltip.AddLine then return end
				tooltip:AddLine("|cFF00FF00aLoot|r")
			end,
		})
		
		LibStub("LibDBIcon-1.0"):Register(_, LDB, miniMapIconSave)
		if L.config.aLoot_minimap then
			LibStub("LibDBIcon-1.0"):Show("aLoot")
		end
			
		-- Slash handler
		SLASH_ALOOT1 = "/aloot"
		SlashCmdList["ALOOT"] = function(msg)
			local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)")
			if ( cmd == "test" ) then
				--lootFrameMixin:ShowFlashFrame(nil, "Test ALootFlashFrame")
				lootFrameMixin:AddFrameToPool(133641, "|cff00bbffTest aLootInfoFrame|r")
			else
				LibStub('AceConfigDialog-3.0'):Open("aLoot")
			end
		end
		
		if IsAddOnLoaded(_) then
			self:UnregisterEvent('ADDON_LOADED')
			self.ADDON_LOADED = nil
		end
	end
end

function lootFrameMixin:ConfigUpdate()
	if aLootFrame.numLootItems and aLootFrame.numLootItems > 0 then
		for i = 1, aLootFrame.numLootItems do
			aLootFrame.slots[i]:SetSize(L.config.iconSize, L.config.iconSize)
			aLootFrame.slots[i].nameTexture:SetTexture(LSM:Fetch(LSM.MediaType.STATUSBAR, L.config.name_texture))
			aLootFrame.slots[i].name:SetFont(LSM:Fetch(LSM.MediaType.FONT, L.config.name_font), L.config.name_fontsize,L.config.name_font_flag)
			if aLootFrame.slots[i].lootName == "Test" then
				OpenSlot(i, true)
				anchorSlots()
			else
				OpenSlot(i)
			end
		end
		anchorSlots()
	end
end

-- iba takto sa da prepisat hodnota config z XML suboru
function lootFrameMixin:UpdateNotifyChange(key, value)
	if key == "aLootFrame" then
		L.config.unlock_lootFrame = value
		print("|cffFF0000LOCK|r and save frame position\n")
		LibStub("AceConfigRegistry-3.0"):NotifyChange("aLoot")
	elseif key == "aLootFlashFrame" then
		L.config.unlock_flashFrame = value
		print("|cffFF0000LOCK|r and save frame position\n")
		LibStub("AceConfigRegistry-3.0"):NotifyChange("aLoot")
	elseif key == "aLootInfoFrame" then
		L.config.unlock_infoFrame = value
		print("|cffFF0000LOCK|r and save frame position\n")
		LibStub("AceConfigRegistry-3.0"):NotifyChange("aLoot")
	elseif key == "openLoot" then
		if aLootFrame:IsShown() and aLootFrame.slots[1] and aLootFrame.slots[1]:IsShown() then
			aLootFrame.slots[1]:Hide()
			aLootFrame.numLootItems = 0
			aLootFrame:Hide()
		else
			OpenSlot(1, true)
			anchorSlots()
		end
	end	
end

function lootFrameMixin:OnEvent(event, ...)
	--print(event)
	if ( event == "ADDON_LOADED" ) then
		OnAddonLoaded(self, ...)
			
	elseif ( event == "UI_ERROR_MESSAGE" ) then
		local errorType, message = ...
		if L.config.flashFrame and aLootFrame.lootReadyIsOpened then
			if MultiCheck(message, ERR_INV_FULL, ERR_ITEM_MAX_COUNT, ERR_LOOT_SLOT, ERR_LOOT_STUNNED, ERR_LOOT_NO_UI, ERR_LOOT_WHILE_INVULNERABLE, ERR_NO_LOOT) then
				if not aLootFlashFrame.Anime:IsPlaying() then
					lootFrameMixin:ShowFlashFrame(nil, "|cffff0000" .. message  .."|r")
					PlaySoundFile(LSM:Fetch(LSM.MediaType.SOUND, L.config.flash_error_sound), "Master")
				end
			end
		end

	elseif ( event == "LOOT_READY" and L.config.speedLoot and not IsModifiedClick("AUTOLOOTTOGGLE") ) then
		OnLootReady(...)
		
	elseif ( event == "LOOT_OPENED" and (not L.config.speedLoot or (L.config.speedLoot and IsModifiedClick("AUTOLOOTTOGGLE")))) then
		OnLootOpened(...)
				
	elseif ( event == "LOOT_SLOT_CLEARED" ) then
		OnLootSlotCleared(...)
				
	elseif ( event == "LOOT_CLOSED" ) then
		OnLootClosed()
						
	elseif ( event == "OPEN_MASTER_LOOT_LIST" ) then
		MasterLooterFrame_Show(_G.LootFrame.selectedLootButton)
		--ToggleDropDownMenu(1, nil, GroupLootDropDown, LootFrame.selectedLootButton, 0, 0)		--LootFrame.selectedSlot
			
	elseif ( event == "UPDATE_MASTER_LOOT_LIST" ) then
		if _G.LootFrame.selectedLootButton then MasterLooterFrame_UpdatePlayers() end
		--MasterLooterFrame_UpdatePlayers()
	end
end

function lootFrameMixin:OnLoad()
	for _, event in pairs({
		"ADDON_LOADED",
		"UI_ERROR_MESSAGE",
		"LOOT_READY",
		"LOOT_OPENED",
		"LOOT_SLOT_CLEARED",
		"LOOT_CLOSED",
		"OPEN_MASTER_LOOT_LIST",
		"UPDATE_MASTER_LOOT_LIST",
	}) do self:RegisterEvent(event) end
--	lootFrameMixin = nil
end
